; --------------------------------------------------------------------------
; Envia, ao display, 8 bits do layout do caracter.
;
; ENTRADA: W contem o layout do caracter
; SAIDA  : Caracter aparece no display selecionado
; --------------------------------------------------------------------------
send     movwf    R13                            ; salva o layout do caracter
         comf     R13,f                          ; lembre-se que o display e' katodo comun

         movlw    0x0F                           ; mascara
         andwf    DISPLAY,f                      ; selecao do display

         bsf      STATUS,RP0                     ; banco 1
         movlw    DSPSEL_BITS                    ; mascara de I/O da porta de selecao
         andwf    DSPSEL_TRIS,f                  ; configura I/O da porta de selecao
         movlw    DSPDATA_BITS                   ; mascara de I/O da porta de dados
         andwf    DSPDATA_TRIS,f                 ; configura I/O da porta de dados
         bcf      STATUS,RP0                     ; banco 0

         movf     DISPLAY,w                      ; mascara de selecao de digito do display
         movwf    DSPSEL_PORT                    ; seleciona um digito

         nop                                     ;
         nop                                     ;

         movf     R13,w                          ; layout do digito
         movwf    DSPDATA_PORT                   ; mostra o digito no display 
         return

; --------------------------------------------------------------------------
; Exibe Hexadecimal no Terminal
; --------------------------------------------------------------------------
shexa  call  HEXAG
       movf  R1,w
       USART_TX
       movf  R0,w
       USART_TX
       return




; --------------------------------------------------------------------------
; CONVERTE BINARIO A DECIMAL
; --------------------------------------------------------------------------
;*** 32 BIT SIGNED BINARY TO DECIMAL ***
;REGA -> DIGITS 1 (MSD) TO 10 (LSD) & DSIGN
;DSIGN = 0 if REGA is positive, 1 if negative
;Return carry set if overflow
;Uses FSR register

bin2dec	clrf	MTEMP		;Reset sign flag
	call	absa		;Make REGA positive
	skpnc
	return			;Overflow

	call	clrdig		;Clear all digits

	movlw	D'32'		;Loop counter
	movwf	MCOUNT

b2dloop	rlf	REGA0,f		;Shift msb into carry           ?     0000.0000 0000.0001 1000.0010 1110.0101
	rlf	REGA1,f
	rlf	REGA2,f
	rlf	REGA3,f         ;                               0     0000.0000 0000.0011 0000.0101 1100.101?

	movlw	DIGIT10
	movwf	FSR		;Pointer to digits
	movlw	D'10'		;10 digits to do
	movwf	DCOUNT

adjlp	rlf	INDF,f		;Shift digit and carry 1 bit left     
	movlw	D'10'
	subwf	INDF,w		;Check and adjust for decimal overflow
	skpnc
	movwf	INDF

	decf	FSR,f		;Next digit
	decfsz	DCOUNT,f
	goto	adjlp

	decfsz	MCOUNT,f	;Next bit
	goto	b2dloop

	btfsc	MTEMP,0		;Check sign
	bsf	DSIGN,0		;Negative
	clrc
	return


;Check sign of REGA and convert negative to positive
;Used by multiply, divide, bin2dec, round

absa	rlf	REGA3,w
	skpc
	return			;Positive

;Set all digits to 0
;Used by bin2dec

clrdig	clrf	DSIGN
	clrf	DIGIT1
	clrf	DIGIT2
	clrf	DIGIT3
	clrf	DIGIT4
	clrf	DIGIT5
	clrf	DIGIT6
	clrf	DIGIT7
	clrf	DIGIT8
	clrf	DIGIT9
	clrf	DIGIT10
	return




; --------------------------------------------------------------------------
; CONVERTE DECIMAL A BINARIO
; --------------------------------------------------------------------------
;*** 32 BIT SIGNED DECIMAL TO BINARY ***
;Decimal DIGIT1 thro DIGIT(X) & DSIGN -> REGA
;Set DSIGN = 0 for positive, DSIGN = 1 for negative values
;Most significant digit in DIGIT1
;Enter this routine with digit count in w register
;Return carry set if overflow
;Uses FSR register

dec2bin	movwf	MTEMP		;Save digit count

	movlw	D'32'		;Outer bit loop counter
	movwf	MCOUNT

d2blp1	movlw	DIGIT1-1	;Set up pointer to MSD
	movwf	FSR
	movf	MTEMP,w		;Inner digit loop counter
	movwf	DCOUNT

	movlw	D'10'
	clrc			;Bring in '0' bit into MSD

d2blp2	incf	FSR,f
	skpnc
	addwf	INDF,f		;Add 10 if '1' bit from prev digit
	rrf	INDF,f		;Shift out LSB of digit

	decfsz	DCOUNT,f	;Next L.S. Digit
	goto	d2blp2

	rrf	REGA3,f		;Shift in carry from digits
	rrf	REGA2,f
	rrf	REGA1,f
	rrf	REGA0,f

	decfsz	MCOUNT,f	;Next bit
	goto	d2blp1

	movf	INDF,w		;Check for overflow
	addlw	0xFF
	skpc
	rlf	REGA3,w
	skpnc
	return

	btfsc	DSIGN,0		;Check result sign
	call	d2bNA		;Negative
	return

d2bNA	movf	REGA3,w		;Save sign in w
	andlw	0x80

	comf	REGA0,f		;2's complement
	comf	REGA1,f
	comf	REGA2,f
	comf	REGA3,f
	incfsz	REGA0,f
	goto	d2bNA1
	incfsz	REGA1,f
	goto	d2bNA1
	incfsz	REGA2,f
	goto	d2bNA1
	incf	REGA3,f
d2bNA1
	incf	MTEMP,f		;flip sign flag
	addwf	REGA3,w		;Return carry set if -2147483648
	return
