; --------------------------------------------------------------------------
; CONFIGURACAO DA INTERRUPCAO EXTERNA (Entra em modo de acerto do relogio)
; --------------------------------------------------------------------------
EXT_config   bcf      FLAGS,ACMOD                ; limpa indicador de modo
             bsf      STATUS,RP0                 ; banco 1
             bsf      TRISB,0                    ; RB0 = INPUT 
             bcf      OPTION_REG,INTEDG          ; interrupcao na descida do sinal
             bcf      STATUS,RP0                 ; banco 0
             bsf      INTCON,PEIE                ; interrupcoes perifericas 
             bsf      INTCON,GIE                 ; habilita geral
             bsf      INTCON,INTE                ; habilita interrupcao em RB0
             return

; --------------------------------------------------------------------------
; CONFIGURACAO DO TIMER 1 PARA TEMPORIZACAO DA MULTIPLEXACAO DOS DISPLAYS
; --------------------------------------------------------------------------
TMR1_config
          bcf     T1CON,TMR1ON                   ; desliga o TIMER 
          movlw   HIGH(mplxtimer)                ; MSB para TMR1 
          movwf   TMR1H                          ; 
          movlw   LOW(mplxtimer)                 ; LSB para TMR1 
          movwf   TMR1L                          ; 
          clrf    T1CON                          ; limpa T1CON 
          bcf     T1CON,T1CKPS1                  ; prescaler 
          bcf     T1CON,T1CKPS0                  ; prescaler 
          bcf     T1CON,TMR1CS                   ; modo temporizador 
          bsf     T1CON,TMR1ON                   ; liga TIMEWK1 
          bsf     STATUS,RP0                     ; banco 1
          bsf     PIE1,TMR1IE                    ; habilita interrupcao de TIMEWK1
          bcf     STATUS,RP0                     ; banco 0
          bsf     INTCON,PEIE                    ; interrupcoes perifericas 
          bsf     INTCON,GIE                     ; habilita geral
          return




; --------------------------------------------------------------------------
; ROTINA DE INTERRUPCAO DE NIVEL 1
; --------------------------------------------------------------------------
FLIH     btfsc   PIR1,TMR1IF                     ; TMR1???
         goto    intTMR1

         btfsc   INTCON,INTF                     ; EXTERNA???
         goto    intEXT

         retfie

; --------------------------------------------------------------------------
; ROTINA DE INTERRUPCAO DO TMR1
; --------------------------------------------------------------------------
intTMR1   bsf     STATUS,RP0                      ; banco 1
          bcf     PIE1,TMR1IE                     ; desabilita interrupcao de TIMER
          bcf     STATUS,RP0                      ; banco 0
          swapf   tmr1svW,f                         ; salva W
          movf    STATUS,w                        ; pega o STATUS
          movwf   tmr1svST                          ; salva o STATUS

          btfss   DISPLAY,CS0                     ; display de unidades???
          goto    selA4                           ; SIM --> 

          btfss   DISPLAY,CS1                     ; display de dezenas
          goto    selA3                           ; SIM --> 

          btfss   DISPLAY,CS2                     ; display de 
          goto    selA2                           ; SIM --> centenas

          btfss   DISPLAY,CS3                     ; display de 
          goto    selA1                           ; SIM --> milhares

          goto    intfim                          ; nenhum display selecionado

selA4     movf    min,w                          ; pega o digito 0
          andlw   0x0F                            ; deixa so o nible 0 (endereco p/ table read)
          call    charset                         ; pega layout na table read
          call    send                            ; envia ao display CS0
          movlw   0x0D                            ; mascara do display CS1
          movwf   DISPLAY                         ; pre-seleciona o display CS1
          goto    intfim                          ; fim

selA3     movf    min,w                          ; pega o digito 0
          movwf   iR1                              ; salva o digito 0
          rrf     iR1,f                            ; empurra 1 bit p/ direita
          rrf     iR1,f                            ; empurra 1 bit p/ direita
          rrf     iR1,f                            ; empurra 1 bit p/ direita
          rrf     iR1,f                            ; empurra 1 bit p/ direita
          movf    iR1,w                            ; pega o resultado do empurra-empurra
          andlw   0x0F                            ; deixa so o valor do nible 1 do digito 0
          call    charset                         ; pega o layout na table read
          call    send                            ; envia para o display CS1
          movlw   0x0B                            ; mascara do dsiplay CS2
          movwf   DISPLAY                         ; pre-seleciona display CS2
          goto    intfim                          ; fim

selA2     movf    hor,w                          ; pega o digito 1
          andlw   0x0F                            ; deixa so o nible 0 (endereco p/ table read)
          call    charset                         ; pega layout na table read
          call    send                            ; envia ao display CS2

          ; Nao deixa mostrar caracter zero no diaplay CS3 porque nele nao ha segmento "f"
          movf   hor,w                           ; pega o digito 1
          andlw  0xF0                             ; deixa so o nible 1
          xorlw  0x00                             ; verifica situacao
          btfsc  STATUS,Z                         ; nible 1 de digito 1 e' zero???
          goto   selcs0                           ; SIM --> nao mostra pq o display CS3 e'
                                                  ;         aleijado segmento "f"
          movlw   0x07                            ; mascara do display CS3
          movwf   DISPLAY                         ; pre-seleciona display CS3
          goto    intfim                          ; fim

selA1     movf    hor,w                          ; pega o digito 1
          movwf   iR1                              ; salva o digito 1
          rrf     iR1,f                            ; empurra 1 bit p/ direita
          rrf     iR1,f                            ; empurra 1 bit p/ direita
          rrf     iR1,f                            ; empurra 1 bit p/ direita
          rrf     iR1,f                            ; empurra 1 bit p/ direita
          movf    iR1,w                            ; pega o resultado do empurra-empurra
          andlw   0x0F                            ; deixa so o valor do nible 1 do digito 1
          call    charset                         ; pega o layout na table read
          call    send                            ; envia para o display CS3
selcs0    movlw   0x0E                            ; mascara do dsiplay CS0
          movwf   DISPLAY                         ; pre-seleciona display CS0

intfim    movlw   HIGH(mplxtimer)                 ; MSB do TIMER1
          movwf   TMR1H                           ; recarrega MSB do TIMER1
          movlw   LOW(mplxtimer)                  ; LSB do TIMER1
          movwf   TMR1L                           ; recarrega LSB do TIMER1
          movf    tmr1svST,w                        ; velho STATUS
          movwf   STATUS                          ; restaura o STATUS
          swapf   tmr1svW,w                         ; restaura W
          bcf     PIR1,TMR1IF                     ; limpa flag de interrupcao
          bsf     STATUS,RP0                      ; banco 1
          bsf     PIE1,TMR1IE                     ; reabilita interrupcao de TIMER1
          bcf     STATUS,RP0                      ; banco 0
          retfie                                  ; fim





; --------------------------------------------------------------------------
; ROTINA DE INTERRUPCAO EXTERNA (RB0)
; Toda vez que esta interrupcao ocorrer, o FLAGS<ACMOD> sera invertido,
; isto e', se o programa estiver no modo normal (relogio), entrara em modo 
; de acerto das horas e minutos. Isso devera ser gerenciado pela
; rotina principal do programa.
; --------------------------------------------------------------------------
intEXT       swapf     extsvW,f                  ; salva W
             movf      STATUS,w                  ; pega o STATUS
             movwf     extsvST                   ; salva o STATUS

intEXT1      movf      BTMODO_PORT,w              ; leitura do botao de modo
             movwf     rgsetx                    ; salva a leitura
             btfss     rgsetx,BTMODO              ; botao ainda pressionado???
             goto      intEXT1                   ; SIM --> espera soltar o botao
   
             bsf       FLAGS,ACMOD               ; liga flag ACMOD

             movf      extsvST,w                 ; velho STATUS
             movwf     STATUS                    ; restaura o STATUS
             swapf     extsvW,w                  ; restaura W
             bcf       INTCON,INTF               ; limpa flag de interrupt em RB0
             bsf       INTCON,INTE               ; habilita interrupcao RB0
             retfie                              ; fim
