; ----------------------------------------------------------------------------
; MODO DEBUG 
; 
; Este modo ganha controle quando o jump JP1 estiver ausente, isto e',
; quando o pino JP1 do port JP1_PORT estiver em nivel alto (+5v).
;
; Modo debug opera com interrupcoes desabilitadas senao a USART nao funciona.
;
; Nesto modo o display nao sera atualizado.
;
; Neste modo nao sera possivel entrar no "modo de acerto" atraves do botoes.
;
; Uma sessao de Hiperterminal de 9600BPS fara conexao com o microcontrolador
; a fim de verificar o estado do RTC DS1302. 
;
; Os seguintes comandos sao aceitos neste modo:
;
; O comando "tssmmhhddmmwwaa00" e' usado para acertar data e hora do RTC.
; O comando "r" ou simplesmente um ENTER exibe data e hora no terminal.
; O comando "k" apaga a tela do terminal.
; O comando "z" retorna ao modo normal (relogio) quando estiver no modo debug.

; ----------------------------------------------------------------------------
          bsf     STATUS,RP0                     ; banco 1
          bcf     PIE1,TMR1IE                    ; desabilita interrupcao de TMR1
          bsf     USART_TRIS,USART_TXPIN         ; I/O da USART - pino TX
          bsf     USART_TRIS,USART_RXPIN         ; I/O da USART - pino RX
          bcf     STATUS,RP0                     ; banco 0

          bcf     INTCON,INTE                    ; desabilita interrupcao em RB0 (modo de acerto)

          USART_CONFIG .9600,.20000000,"A","N","C"
          radix hex

          movlw   .100
          call    delay_ms
          call    CLS
          call    msg001
          call    NEWLINE
          call    PROMPT
          movlw   .200                            ; tempo
          call    delay_us                        ; 

; ---------------------------------------------------------------------------------------------------------
; MODO DE COMANDOS
; ----------------------------------------------------------------------------------------------------------
modo_comandos

ler_serial
           btfsc  FLAGS,DS1302_KEY               ; DS1302 ocupado???
           goto   ler_serial                     ; SIM -->
           movlw  rs232buff                      ; NAO -->
           movwf  FSR                            ; inicio do buffer
ler_serial1        
           USART_RX                              ; le a porta serial
           movwf  INDF                           ; salva caracter lido
           incf   FSR,f
           xorlw  0x0D                           ; fim de linha?
           btfss  STATUS,Z                       ; sim, ve se tem comando valido na linha
           goto   ler_serial1                    ; nao, le a porta ate o fim...
           call   NEWLINE
           call   PROMPT
 
; ----------------------------------------------------------------------------------------------------------
; COMANDO: t ---> acerta as horas
; ----------------------------------------------------------------------------------------------------------
v_time     movlw  "t"
           xorwf  rs232buff,w
           btfss  STATUS,Z
           goto   v_ler
           movlw  0x0D
           xorwf  rs232buff+1,w
           btfsc  STATUS,Z
           goto   v_ler
           movlw  rs232buff+1
           movwf  FSR

clrind     movf   INDF,w
           xorlw  0x0D
           btfsc  STATUS,Z
           goto   convert
           movlw  0x0F
           andwf  INDF,f
           incf   FSR,f
           goto   clrind
         

convert    movlw  rs232buff+1                     ; SEGUNDOS
           movwf  FSR
           call   rleft                          ; primeiro digito - rola LSB para o MSB
           movf   rs232buff+1,w                   ; pega primeiro digito
           movwf  rs232buff+0                     ; coloca na posicao desejada
           movf   rs232buff+2,w                   ; pega segundo digito
           iorwf  rs232buff+0,f                   ; junta LSB com MSB

           movlw  rs232buff+3                     ; MINUTOS
           movwf  FSR                       
           call   rleft                          ; primeiro digito - rola LSB para o MSB
           movf   rs232buff+3,w                   ; pega primeiro digito
           movwf  rs232buff+1                     ; coloca na posicao desejada
           movf   rs232buff+4,w                   ; pega segundo digito
           iorwf  rs232buff+1,f                   ; junta LSB com MSB

           movlw  rs232buff+5                     ; HORAS
           movwf  FSR                       
           call   rleft                          ; primeiro digito - rola LSB para o MSB
           movf   rs232buff+5,w                   ; pega primeiro digito
           movwf  rs232buff+2                     ; coloca na posicao desejada
           movf   rs232buff+6,w                   ; pega segundo digito
           iorwf  rs232buff+2,f                   ; junta LSB com MSB

           movlw  rs232buff+7                     ; DIA DO MES
           movwf  FSR                       
           call   rleft                          ; primeiro digito - rola LSB para o MSB
           movf   rs232buff+7,w                   ; pega primeiro digito
           movwf  rs232buff+3                     ; coloca na posicao desejada
           movf   rs232buff+8,w                   ; pega segundo digito
           iorwf  rs232buff+3,f                   ; junta LSB com MSB

           movlw  rs232buff+9                     ; MES
           movwf  FSR                       
           call   rleft                          ; primeiro digito - rola LSB para o MSB
           movf   rs232buff+9,w                   ; pega primeiro digito
           movwf  rs232buff+4                     ; coloca na posicao desejada
           movf   rs232buff+.10,w                 ; pega segundo digito
           iorwf  rs232buff+4,f                   ; junta LSB com MSB

           movlw  rs232buff+.11                   ; DIA DA SEMANA
           movwf  FSR                       
           call   rleft                          ; primeiro digito - rola LSB para o MSB
           movf   rs232buff+.11,w                 ; pega primeiro digito
           movwf  rs232buff+5                     ; coloca na posicao desejada
           movf   rs232buff+.12,w                 ; pega segundo digito
           iorwf  rs232buff+5,f                   ; junta LSB com MSB

           movlw  rs232buff+.13                   ; ANO
           movwf  FSR                       
           call   rleft                          ; primeiro digito - rola LSB para o MSB
           movf   rs232buff+.13,w                 ; pega primeiro digito
           movwf  rs232buff+6                     ; coloca na posicao desejada
           movf   rs232buff+.14,w                 ; pega segundo digito
           iorwf  rs232buff+6,f                   ; junta LSB com MSB

           clrf   rs232buff+7                     ; wp=0

           movlw  0x0D
           movwf  rs232buff+8                     ; fim do buffer

           movlw  rs232buff
           movwf  FSR
           call   DS1302_SET_TIME

           call   NEWLINE
           call   PROMPT
           goto   ler_serial

rleft      rlf    INDF,f
           rlf    INDF,f
           rlf    INDF,f
           rlf    INDF,f
           return
; ----------------------------------------------------------------------------------------------------------
; COMANDO: r ---> ler os 9 registradores
; ----------------------------------------------------------------------------------------------------------
v_ler      movlw  "r"
           xorwf  rs232buff,w
           btfss  STATUS,Z
           goto   v_cls
           movlw  0x0D
           xorwf  rs232buff+1,w
           btfss  STATUS,Z
           goto   v_cls

v_ler1     movlw  rs232buff 
           movwf  FSR
           call   DS1302_READ_BURST
           call   NEWLINE
           call   PROMPT


           movlw  "["
           USART_TX
           movf   rs232buff+5,w
           call   shresp
           movlw  "]"
           USART_TX
           movlw  " "
           USART_TX


           movf   rs232buff+3,w
           call   shresp
           movlw  "/"
           USART_TX
           movf   rs232buff+4,w
           call   shresp
           movlw  "/"
           USART_TX
           movf   rs232buff+6,w
           call   shresp
           movlw  " "
           USART_TX
           movlw  "*"
           USART_TX
           movlw  " "
           USART_TX

           movf   rs232buff+2,w
           call   shresp
           movlw  ":"
           USART_TX
           movf   rs232buff+1,w
           call   shresp
           movlw  ":"
           USART_TX
           movf   rs232buff+0,w
           call   shresp
           movlw  " "
           USART_TX

v_ler2     call   NEWLINE
           call   PROMPT
           goto   ler_serial
; ----------------------------------------------------------------------------------------------------------
; COMANDO: k ---> limpa a tela do terminal
; ----------------------------------------------------------------------------------------------------------
v_cls      movlw  "k"
           xorwf  rs232buff,w
           btfss  STATUS,Z
           goto   v_dspl
           movlw  0x0D
           xorwf  rs232buff+1,w
           btfss  STATUS,Z
           goto   v_xxx
           call   CLS
           call   PROMPT
           goto   ler_serial
; ----------------------------------------------------------------------------------------------------------
; COMANDO: z ---> vai para o modo normal
; ----------------------------------------------------------------------------------------------------------
v_dspl     movlw  "z"
           xorwf  rs232buff,w
           btfss  STATUS,Z
           goto   v_xxx
           movlw  0x0D
           xorwf  rs232buff+1,w
           btfss  STATUS,Z
           goto   v_xxx

           call   TMR1_config                    ; reconfigura interrupcao de TMR1
           call   EXT_config                     ; reconfigura interrupcao externa

           goto   modo_normal




; ----------------------------------------------------------------------------------------------------------
; COMANDO INVALIDO
; ---------------------------------------------------------------------------------------------------------- 
v_xxx      goto   v_ler1
           call   msg002                         ; "COMANDO INVALIDO"
           call   NEWLINE
           call   PROMPT
           goto   ler_serial

; ----------------------------------------------------------------------------------------------------------
;  
; ----------------------------------------------------------------------------------------------------------
clsbuff    movlw  rs232buff
           movwf  FSR
           movlw  .60
           movwf  R1
           movlw  0x00
clsbuff1   movwf  INDF
           incf   FSR,f
           decfsz R1,f
           goto   clsbuff1
           return



; ----------------------------------------------------------------------------------------------------------
;  
; ----------------------------------------------------------------------------------------------------------
shresp     call   HEXAG
           movf   R1,w
           USART_TX
           movf   R0,w
           USART_TX
           return